package com.huimv.eartag2.manage2.utils;

import java.util.Arrays;
import java.util.List;
import java.util.Random;

public class Filter {
    private final List<Double> data;
    private final List<Double> deal;
    private final Double rand;
    private final Random random = new Random(0);

    public Filter(List<Double> data, Double rand) {
        this.data = data;
        this.rand = rand;

        Double[] zeros = new Double[data.size()];
        Arrays.fill(zeros, .0);
        deal = Arrays.asList(zeros);
    }

    private double wave(Double num) {
        return num + (random.nextFloat() - 0.5) * rand;
    }

    private int loc(int start, int end) {
        end = Math.min(end, data.size());
        int ix = start;
        Double mx = data.get(start);
        for (int i = start + 1; i < end; ++i) {
            if (data.get(i) > mx) {
                ix = i;
                mx = data.get(i);
            }
        }
        return ix;
    }

    private int peak(int start) {
        for (int i = start; i < data.size(); ++i) {
            if (deal.get(i) > .0) return i;
        }
        return -1;
    }

    private void bridge(int left, int right) {
        double gap = (deal.get(right) - deal.get(left)) / (right - left);
        for (int i = left + 1; i < right; ++i) deal.set(i, wave(deal.get(left) + (i - left) * gap));
    }

    public List<Double> max(int step, int win, double min, double max) {
        for (int pos = 0; pos < data.size(); pos += step) {
            int idx = loc(pos, pos + win);
            if (data.get(idx) < min) deal.set(idx, wave(min));
            else if (data.get(idx) > max) deal.set(idx, wave(max));
            else deal.set(idx, data.get(idx));
        }

        // left side
        int pos = peak(0);
        if (pos != -1 && pos != 0) for (int idx = pos; idx > 0; --idx) deal.set(idx - 1, wave(deal.get(idx)));
        // center
        while (pos != -1 && pos < data.size()) {
            int idx = peak(pos + 1);
            if (idx == -1) break;
            bridge(pos, idx);
            pos = idx;
        }
        // right side
        if (pos != -1 && pos < data.size() - 1)
            for (int idx = pos; idx < data.size() - 1; ++idx) {
                deal.set(idx + 1, wave(deal.get(idx)));
            }

        return deal;
    }

    public void show() {
        System.out.println("data = " + data);
        System.out.println("deal = " + deal);
    }
}
